<?php

namespace Drupal\Tests\role_delegation\Functional;

use Drupal\Tests\BrowserTestBase;

/**
 * Functional tests for assigning roles.
 *
 * @group role_delegation
 */
class RoleAssignTest extends BrowserTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = ['user', 'role_delegation'];

  /**
   * Ensure we can only see the roles we have permission to assign.
   */
  public function testRoleAccess() {
    $rid1 = $this->drupalCreateRole([]);
    $rid2 = $this->drupalCreateRole([]);
    $rid3 = $this->drupalCreateRole([]);

    $current_user = $this->drupalCreateUser([
      sprintf('assign %s role', $rid1),
      sprintf('assign %s role', $rid2),
    ]);
    $this->drupalLogin($current_user);

    // Only 2 of the 3 roles appear on any user edit page.
    $account = $this->drupalCreateUser();
    $this->drupalGet(sprintf('/user/%s/roles', $account->id()));
    $this->assertFieldByName(sprintf('role_change[%s]', $rid1));
    $this->assertFieldByName(sprintf('role_change[%s]', $rid2));
    $this->assertNoFieldByName(sprintf('role_change[%s]', $rid3));
  }

  /**
   * Test that we can assign roles we have access to.
   */
  public function testRoleAssign() {
    $user_storage = \Drupal::entityTypeManager()->getStorage('user');
    // Create a role and login as a user with the permission to assign it.
    $rid1 = $this->drupalCreateRole([]);
    $current_user = $this->drupalCreateUser([sprintf('assign %s role', $rid1)]);
    $this->drupalLogin($current_user);

    // Go to the users roles edit page.
    $account = $this->drupalCreateUser();
    $this->drupalGet(sprintf('/user/%s/roles', $account->id()));

    // The form element field id and name.
    $field_id = sprintf('edit-role-change-%s', $rid1);
    $field_name = sprintf('role_change[%s]', $rid1);

    // Ensure its disabled by default.
    $this->assertSession()->checkboxNotChecked($field_id);
    self::assertFalse($account->hasPermission(sprintf('assign %s role', $rid1)), 'Role is not assigned by default.');
    self::assertFalse($account->hasRole($rid1), 'Role is not assigned by default.');

    // Assign the role and ensure its now checked and assigned.
    $this->drupalPostForm(NULL, [$field_name => $rid1], 'Save');
    $this->assertSession()->checkboxChecked($field_id);
    $user_storage->resetCache();
    $account = $user_storage->load($account->id());
    self::assertTrue($account->hasRole($rid1), 'Role has been granted');

    // Revoke the role.
    $this->drupalPostForm(NULL, [$field_name => 0], 'Save');
    $this->assertSession()->checkboxNotChecked($field_id);
    $user_storage->resetCache();
    $account = $user_storage->load($account->id());
    self::assertFalse($account->hasRole($rid1), 'Role has been revoked.');
  }

  /**
   * Test that the user has access to the role delegation page.
   */
  public function testRoleDelegationPageAccess() {
    $regular_user = $this->drupalCreateUser();

    // Anonymous users can never access the roles page.
    $this->drupalGet(sprintf('/user/%s/roles', $regular_user->id()));
    $this->assertSession()->statusCodeEquals(403);

    // Users with "administer permissions" cannot view the page, they must use
    // the normal user edit page or also "have assign all roles".
    $account = $this->createUser(['administer permissions']);
    $this->drupalLogin($account);
    $this->drupalGet(sprintf('/user/%s/roles', $regular_user->id()));
    $this->assertSession()->statusCodeEquals(403);

    // Users with a custom "assign %custom role" permission should be able to
    // see the role admin page.
    $role = $this->createRole([]);
    $account = $this->createUser([sprintf('assign %s role', $role)]);
    $this->drupalLogin($account);
    $this->drupalGet(sprintf('/user/%s/roles', $regular_user->id()));
    $this->assertSession()->statusCodeEquals(200);

    // Users with 'assign all roles' can view the page.
    $account = $this->createUser(['assign all roles']);
    $this->drupalLogin($account);
    $this->drupalGet(sprintf('/user/%s/roles', $regular_user->id()));
    $this->assertSession()->statusCodeEquals(200);
  }

}
